@@ -6,16 +6,16 @@
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU Library General Public License for more details.
 #
-# You should have received a copy of the GNU Library General 
-# Public License along with this library; if not, write to the 
-# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
+# You should have received a copy of the GNU Library General
+# Public License along with this library; if not, write to the
+# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 # MA  02111-1307  USA
 
 # Copyrights (C)
-# for this R-port: 
+# for this R-port:
 #   1999 - 2007, Diethelm Wuertz, GPL
 #   Diethelm Wuertz <wuertz@itp.phys.ethz.ch>
 #   info@rmetrics.org
@@ -43,37 +43,37 @@
 
 
 gpdTailPlot =
-function(object, plottype = c("xy", "x", "y", ""), doplot = TRUE, extend = 1.5, 
+function(object, plottype = c("xy", "x", "y", ""), doplot = TRUE, extend = 1.5,
 labels = TRUE, ...)
 {   # A function implemented by Diethelm Wuertz
 
     # Description:
     #   Plots tail estimate from GPD model
-    
+
     # Arguments:
     #   object - an object of class 'fGPDFIT'
-    
+
     # Note:
     #   Code partly copied from R package evir
-    
+
     # Example:
     #   object = gpdFit(as.timeSeries(data(danishClaims)), u = 10)
     #   gpdTailPlot(object)
-    
-    # FUNCTION:   
-    
+
+    # FUNCTION:
+
     # Settings:
     threshold = object@fit$threshold
     x = as.vector(object@data$x)
     data = x[x > threshold]
     xi = as.numeric(object@fit$par.ests["xi"])
     beta = as.numeric(object@fit$par.ests["beta"])
-    
+
     # Points:
     plotmin = threshold
     plotmax = max(data) * max(1, extend)
     z = qgpd(seq(from = 0, to = 1, length = 501), xi, threshold, beta)
-    z = pmax(pmin(z, plotmax), plotmin)    
+    z = pmax(pmin(z, plotmax), plotmin)
     invProb = 1 - length(data)/length(x)
     ypoints = invProb*(1-ppoints(sort(data)))
     y = invProb*(1-pgpd(z, xi, threshold, beta))
@@ -82,13 +82,13 @@ labels = TRUE, ...)
     shape = xi
     scale = beta * invProb^xi
     location = threshold - (scale*(invProb^(- xi)-1))/xi
-    
+
     # Show Plot:
     if (doplot) {
         # Plot
-        plot(sort(data), ypoints, xlim = range(plotmin, plotmax), 
+        plot(sort(data), ypoints, xlim = range(plotmin, plotmax),
              ylim = range(ypoints, y, na.rm = TRUE), col = "steelblue",
-             pch = 19, xlab = "", ylab = "", log = plottype[1], 
+             pch = 19, xlab = "", ylab = "", log = plottype[1],
              axes = TRUE, ...)
         lines(z[y >= 0], y[y >= 0])
         grid()
@@ -105,26 +105,26 @@ labels = TRUE, ...)
             title(main = "Tail Estimate Plot")
         }
     }
-    
+
     # Object:
     object@fit$n = length(x)
     object@fit$data = object@data$exceedances
     object@fit$n.exceed = length(object@fit$data)
     if(object@method[2] == "mle") {
-        object@fit$method = "ml" 
+        object@fit$method = "ml"
     } else {
         object@fit$method = ""
     }
-    
+
     # Last Fit:
     lastfit = object@fit
     class(lastfit) = "gpd"
-    
+
     # Result:
     ans = list(lastfit = lastfit, type = "tail", dist = "gpd",
-        plotmin = plotmin, plotmax = plotmax, alog = plottype[1], 
+        plotmin = plotmin, plotmax = plotmax, alog = plottype[1],
         location = location, shape = shape, scale = scale)
-    
+
     # Return Value:
     invisible(ans)
 }
@@ -138,48 +138,48 @@ function(x, p = 0.99, ci = 0.95, type = c("likelihood", "wald"), like.num = 50)
 {   # A function implemented by Diethelm Wuertz
 
     # Description:
-    #   Adds Expected Shortfall Estimates to a GPD Plot 
-    
+    #   Adds Expected Shortfall Estimates to a GPD Plot
+
     # Arguments:
     #   x  - an object of class 'gpdFit'
     #   pp - the probability level
-    
+
     # Example:
     #   par(mfrow=c(1,1)); x=as.timeSeries(data(danishClaims))
     #   tp = gpdTailPlot(gpdFit(x, 10)); gpdQPlot(tp)
-    
-    # FUNCTION: 
-    
+
+    # FUNCTION:
+
     # Check Argument:
     par(new = TRUE)
     ci.p = ci
     pp = p
     ci.type = type[1]
 
-    # A copy from evir:  
-    if (x$dist != "gpd") 
+    # A copy from evir:
+    if (x$dist != "gpd")
         stop("This function is used only with GPD curves")
-    if (length(pp) > 1) 
+    if (length(pp) > 1)
         stop("One probability at a time please")
-        
+
     threshold = x$lastfit$threshold
     par.ests = x$lastfit$par.ests
     xihat = par.ests["xi"]
     betahat = par.ests["beta"]
     varcov = x$lastfit$varcov
     p.less.thresh = x$lastfit$p.less.thresh
-    
+
     lambda = 1
     if (x$type == "tail") lambda = 1/(1 - p.less.thresh)
     a = lambda * (1 - pp)
     gfunc = function(a, xihat) (a^(-xihat) - 1)/xihat
-    gfunc.deriv = function(a, xihat) (-(a^(-xihat) - 1)/xihat - 
+    gfunc.deriv = function(a, xihat) (-(a^(-xihat) - 1)/xihat -
         a^(-xihat) * logb(a))/xihat
     q = q.keep = threshold + betahat * gfunc(a, xihat)
     # if (q < x$plotmax) abline(v = q, lty = 2)
     out = as.numeric(q)
     if (ci.type == "wald") {
-        if (class(x$lastfit) != "gpd") 
+        if (class(x$lastfit) != "gpd")
             stop("Wald method requires model be fitted with gpd (not pot)")
         scaling = threshold
         betahat = betahat/scaling
@@ -190,7 +190,7 @@ function(x, p = 0.99, ci = 0.95, type = c("likelihood", "wald"), like.num = 50)
         term2 = xivar * (betahat^2) * (gfunc.deriv(a, xihat))^2
         term3 = 2 * covar * betavar * gfunc(a, xihat) * gfunc.deriv(a, xihat)
         qvar = term1 + term2 + term3
-        if (qvar < 0) 
+        if (qvar < 0)
             stop("Negative estimate of quantile variance")
         qse = scaling * sqrt(qvar)
         qq = qnorm(1 - (1 - ci.p)/2)
@@ -202,11 +202,11 @@ function(x, p = 0.99, ci = 0.95, type = c("likelihood", "wald"), like.num = 50)
         names(out) = c("Lower CI", "Estimate", "Std.Err", "Upper CI")
     }
     if (ci.type == "likelihood") {
-        parloglik = 
+        parloglik =
         function(theta, tmp, a, threshold, xpi) {
-            beta = (theta * (xpi - threshold))/(a^(-theta) - 
+            beta = (theta * (xpi - threshold))/(a^(-theta) -
                 1)
-            if ((beta <= 0) || ((theta <= 0) && (max(tmp) > (-beta/theta)))) 
+            if ((beta <= 0) || ((theta <= 0) && (max(tmp) > (-beta/theta))))
                 f = 1e+06
             else {
                 y = logb(1 + (theta * tmp)/beta)
@@ -218,12 +218,12 @@ function(x, p = 0.99, ci = 0.95, type = c("likelihood", "wald"), like.num = 50)
         }
         theta = xihat
         parmax = NULL
-        xp = exp(seq(from = logb(threshold), to = logb(x$plotmax), 
+        xp = exp(seq(from = logb(threshold), to = logb(x$plotmax),
             length = like.num))
         excess = as.numeric(x$lastfit$data - threshold)
         for (i in 1:length(xp)) {
-            fit2 = optim(theta, parloglik, method = "BFGS", 
-                hessian = FALSE, tmp = excess, a = a, threshold = threshold, 
+            fit2 = optim(theta, parloglik, method = "BFGS",
+                hessian = FALSE, tmp = excess, a = a, threshold = threshold,
                 xpi = xp[i])
             parmax = rbind(parmax, fit2$value)
         }
@@ -236,10 +236,10 @@ function(x, p = 0.99, ci = 0.95, type = c("likelihood", "wald"), like.num = 50)
         par(new = TRUE)
         dolog = ""
         if (x$alog == "xy" || x$alog == "x") dolog = "x"
-        plot(xp, parmax, type = "n", xlab = "", ylab = "", axes = FALSE, 
-            xlim = range(x$plotmin, x$plotmax), 
+        plot(xp, parmax, type = "n", xlab = "", ylab = "", axes = FALSE,
+            xlim = range(x$plotmin, x$plotmax),
             ylim = range(overallmax, crit), log = dolog)
-        axis(4, at = overallmax - qchisq(c(0.95, 0.99), 1)/2, 
+        axis(4, at = overallmax - qchisq(c(0.95, 0.99), 1)/2,
             labels = c("95", "99"), tick = TRUE)
         aalpha = qchisq(ci.p, 1)
         abline(h = overallmax - aalpha/2, lty = 2, col = 2)
@@ -247,13 +247,13 @@ function(x, p = 0.99, ci = 0.95, type = c("likelihood", "wald"), like.num = 50)
         smth = spline(xp[cond], parmax[cond], n = 200)
         lines(smth, lty = 2, col = 2)
         ci = smth$x[smth$y > overallmax - aalpha/2]
-        
+
         abline(v = q.keep, lty = 2)
-        
+
         out = c(min(ci), q, max(ci))
         names(out) = c("Lower CI", "Estimate", "Upper CI")
     }
-    
+
     # Return Value:
     out
 }
@@ -267,30 +267,30 @@ function(x, p = 0.99, ci = 0.95, like.num = 50)
 {   # A function implemented by Diethelm Wuertz
 
     # Description:
-    #   Adds Expected Shortfall Estimates to a GPD Plot 
-    
+    #   Adds Expected Shortfall Estimates to a GPD Plot
+
     # Arguments:
     #   x  - an object of class 'gpdFit'
-    #   p - the desired probability for expected shortfall 
+    #   p - the desired probability for expected shortfall
     #       estimate (e.g. 0.99 for the 99th percentile)
-    #   ci - probability for confidence interval (must be 
+    #   ci - probability for confidence interval (must be
     #       less than 0.999)
     #   like.num - number of times to evaluate profile likelihood
-    
+
     # FUNCTION:
 
     # Settings:
     par(new = TRUE)
     pp = p
-    ci.p = ci  
+    ci.p = ci
     object = x
-    
+
     # A copy from evir:
     if(x$dist != "gpd")
         stop("This function is used only with GPD curves")
     if(length(pp) > 1)
         stop("One probability at a time please")
-       
+
     threshold = x$lastfit$threshold
     par.ests = x$lastfit$par.ests
     xihat = par.ests["xi"]
@@ -298,8 +298,8 @@ function(x, p = 0.99, ci = 0.95, like.num = 50)
     varcov = x$lastfit$varcov
     p.less.thresh = x$lastfit$p.less.thresh
     lambda = 1
-    
-    # if (x$type == "tail") 
+
+    # if (x$type == "tail")
     lambda = 1/(1 - p.less.thresh)
     a = lambda * (1 - pp)
     gfunc = function(a, xihat) (a^( - xihat) - 1) / xihat
@@ -307,7 +307,7 @@ function(x, p = 0.99, ci = 0.95, like.num = 50)
     s = s.keep = q + (betahat + xihat * (q - threshold))/(1 - xihat)
     # if (s < x$plotmax) abline(v = s, lty = 2)
     out = as.numeric(s)
-    
+
     parloglik = function(theta, tmp, a, threshold, xpi)
     {
         beta = ((1 - theta) * (xpi - threshold)) /
@@ -321,47 +321,47 @@ function(x, p = 0.99, ci = 0.95, like.num = 50)
         }
         f
     }
-    
+
     theta = xihat
     parmax = NULL
     xp = exp(seq(from = log(threshold), to = log(x$plotmax),
         length = like.num))
     excess = as.numeric(x$lastfit$data - threshold)
-    
+
     for (i in 1:length(xp)) {
         fit2 = optim(theta, parloglik, method = "BFGS", hessian = FALSE,
             tmp = excess, a = a, threshold = threshold, xpi = xp[i])
         parmax = rbind(parmax, fit2$value)
     }
-    
+
     parmax =  - parmax
     overallmax =  - parloglik(xihat, excess, a, threshold, s)
     crit = overallmax - qchisq(0.999, 1)/2
     cond = parmax > crit
     xp = xp[cond]
     parmax = parmax[cond]
-    
+
     dolog = ""
     if(x$alog == "xy" || x$alog == "x") dolog = "x"
     par(new = TRUE)
-    plot(xp, parmax, type = "n", xlab = "", ylab = "", axes = FALSE, 
-         xlim = range(x$plotmin, x$plotmax), 
+    plot(xp, parmax, type = "n", xlab = "", ylab = "", axes = FALSE,
+         xlim = range(x$plotmin, x$plotmax),
          ylim = range(overallmax, crit), log = dolog)
     axis(4, at = overallmax - qchisq(c(0.95, 0.99), 1)/2,
          labels = c("95", "99"), tick = TRUE)
-        
+
     aalpha = qchisq(ci.p, 1)
     abline(h = overallmax - aalpha/2, lty = 2, col = 2)
     cond = !is.na(xp) & !is.na(parmax)
     smth = spline(xp[cond], parmax[cond], n = 200)
     lines(smth, lty = 2, col = 2)
     ci = smth$x[smth$y > overallmax - aalpha/2]
-    
+
     abline(v = s.keep, lty = 2)
-    
+
     out = c(min(ci), s, max(ci))
     names(out) = c("Lower CI", "Estimate", "Upper CI")
-    
+
     # Return Value:
     out
 }
@@ -377,14 +377,14 @@ doplot = TRUE, plottype = c("normal", "reverse"), labels = TRUE, ...)
 
     # Description:
     #   Plots of GPD Tail Estimate of a High Quantile
-    
+
     # Example:
     #   Danish = as.timeSeries(data(danishClaims))
     #   gpdquantPlot(Danish)
-    
+
     # Note:
     #   Code partly copied from R package evir
-    
+
     # FUNCTION:
 
     # Settings:
@@ -398,10 +398,10 @@ doplot = TRUE, plottype = c("normal", "reverse"), labels = TRUE, ...)
     .quantFit = function(nex, data) {
         prob = 1 - nex/length(as.vector(data))
         fit = gpdFit(data, u = quantile(data, prob))@fit
-        c(fit$threshold, fit$par.ests, fit$par.ses, 
+        c(fit$threshold, fit$par.ests, fit$par.ses,
             as.vector(fit$varcov)[c(1,4,2)])
     }
-    
+
     # Compute:
     mat = apply(as.matrix(exceed), 1, .quantFit, data = data)
     thresh = mat[1, ]
@@ -413,7 +413,7 @@ doplot = TRUE, plottype = c("normal", "reverse"), labels = TRUE, ...)
     qest = thresh + betahat * gfunc(a, xihat)
     l = u = qest
     yrange = range(qest)
-    
+
     # Add Confidence Intervals:
     if (ci) {
         qq = qnorm(1 - (1 - ci)/2)
@@ -439,12 +439,12 @@ doplot = TRUE, plottype = c("normal", "reverse"), labels = TRUE, ...)
         l = qest - qse * qq
         yrange = range(qest, u, l)
     }
-    
+
     # Result matrix:
     mat = rbind(thresh, qest, exceed, l, u)
     rownames(mat) = c("threshold", "qest", "exceedances", "lower", "upper")
     colnames(mat) = paste(1:dim(mat)[2])
-    
+
     # Plot:
     if (doplot) {
         if (plottype[1] == "normal") {
@@ -469,16 +469,16 @@ doplot = TRUE, plottype = c("normal", "reverse"), labels = TRUE, ...)
         }
         p = round(p, 3)
         ci = round(ci, 3)
-        text = paste("p =", p, "| ci =", ci, "| start =", 
+        text = paste("p =", p, "| ci =", ci, "| start =",
             start, "| end =", end )
         mtext(text, side = 4, adj = 0, cex = 0.7)
     }
-    
+
     # Add Attributes:
     mat = t(mat)
-    attr(mat, "control") = data.frame(cbind(p = p, ci = ci, 
+    attr(mat, "control") = data.frame(cbind(p = p, ci = ci,
         start = start, end = end), row.names = "")
-    
+
     # Return Value:
     invisible(mat)
 }
@@ -488,29 +488,29 @@ doplot = TRUE, plottype = c("normal", "reverse"), labels = TRUE, ...)
 
 
 gpdShapePlot =
-function(x, ci = 0.95, models = 30, start = 15, end = 500,  
+function(x, ci = 0.95, models = 30, start = 15, end = 500,
 doplot = TRUE, plottype = c("normal", "reverse"), labels = TRUE, ...)
 {   # A function implemented by Diethelm Wuertz
 
     # Description:
     #   Plots for GPD Shape Parameter
-    
+
     # Example:
-    
+
     # FUNCTION:
 
     # Settings:
     data = as.vector(x)
     X = trunc(seq(from = min(end, length(data)), to = start, length = models))
-    
+
     # Internal Function:
     .shapeFit = function(nex, data) {
         prob = 1 - nex/length(as.vector(data))
-        fit = gpdFit(data, u = quantile(data, prob), 
+        fit = gpdFit(data, u = quantile(data, prob),
             information = "expected")@fit
-        c(fit$threshold, fit$par.ests[1], fit$par.ses[1]) 
+        c(fit$threshold, fit$par.ests[1], fit$par.ses[1])
     }
-    
+
     # Result Matrix:
     mat = apply(as.matrix(X), 1, .shapeFit, data = data)
     mat = rbind(mat, X)
@@ -552,12 +552,12 @@ doplot = TRUE, plottype = c("normal", "reverse"), labels = TRUE, ...)
         text = paste("ci =", ci, "| start =", start, "| end =", end )
         mtext(text, side = 4, adj = 0, cex = 0.7)
     }
-    
+
     # Add Attributes:
-    attr(mat, "control") = data.frame(cbind(ci = ci, 
+    attr(mat, "control") = data.frame(cbind(ci = ci,
         start = start, end = end), row.names = "")
     mat = t(mat)
-    
+
     # Return Value:
     invisible(mat)
 }
@@ -566,35 +566,35 @@ doplot = TRUE, plottype = c("normal", "reverse"), labels = TRUE, ...)
 # ------------------------------------------------------------------------------
 
 
-gpdRiskMeasures = 
+gpdRiskMeasures =
 function(object, prob = c(0.99, 0.995, 0.999, 0.9995, 0.9999))
 {   # A function implemented by Diethelm Wuertz
 
     # Description:
     #   Calculates Quantiles and Expected Shortfalls
-    
+
     # Arguments:
     #   x  - an object of class 'gpdFit'
-    #   prob - a numeric value or vector of probability levels 
-    
+    #   prob - a numeric value or vector of probability levels
+
     # FUNCTION:
-    
+
     # Settings:
     u = object@parameter$u
     par.ests = object@fit$par.ests
     xi = par.ests["xi"]
     beta = par.ests["beta"]
     lambda = 1/(1 - object@fit$prob)
-    
-    # Quantile Risk:
-    q = u + (beta * ((lambda * (1 - prob))^( - xi) - 1))/xi 
 
-    # Shortfall Risk:  
-    es = (q * (1 + (beta - xi * u)/q)) / (1 - xi) 
-  
+    # Quantile Risk:
+    q = u + (beta * ((lambda * (1 - prob))^( - xi) - 1))/xi
+
+    # Shortfall Risk:
+    es = (q * (1 + (beta - xi * u)/q)) / (1 - xi)
+
     # Risk Matrix:
     ans = data.frame(p = prob, quantile = q, shortfall = es)
-    
+
     # Return Value:
     ans
 }
@@ -603,25 +603,27 @@ function(object, prob = c(0.99, 0.995, 0.999, 0.9995, 0.9999))
 ################################################################################
 
 
-tailPlot =
-function(object, p = 0.99, ci = 0.95, nLLH = 25, extend = 1.5, labels = TRUE, ...)
+tailPlot <-
+    function(object, p = 0.99, ci = 0.95,
+             nLLH = 25, extend = 1.5,
+             grid = TRUE, labels = TRUE, ...)
 {   # A function implemented by Diethelm Wuertz
 
     # Description:
     #   Plots GPD VaR and Expected Shortfall risk
-    
+
     # Arguments:
     #   object - an object of class 'fGPDFIT'
-    
+
     # Note:
     #   Code partly copied from R package evir
-    
+
     # Example:
     #   object = gpdFit(as.timeSeries(data(danishClaims)), u = 10)
     #   gpdTailPlot(object)
-    
-    # FUNCTION:   
-    
+
+    # FUNCTION:
+
     # Settings:
     ci.p = p
     pp = p
@@ -631,12 +633,12 @@ function(object, p = 0.99, ci = 0.95, nLLH = 25, extend = 1.5, labels = TRUE, ..
     data = x[x > threshold]
     xi = as.numeric(object@fit$par.ests["xi"])
     beta = as.numeric(object@fit$par.ests["beta"])
-    
+
     # Points:
     plotmin = threshold
     plotmax = max(data) * max(1, extend)
     z = qgpd(seq(from = 0, to = 1, length = 501), xi, threshold, beta)
-    z = pmax(pmin(z, plotmax), plotmin)    
+    z = pmax(pmin(z, plotmax), plotmin)
     invProb = 1 - length(data)/length(x)
     ypoints = invProb*(1-ppoints(sort(data)))
     y = invProb*(1-pgpd(z, xi, threshold, beta))
@@ -645,14 +647,15 @@ function(object, p = 0.99, ci = 0.95, nLLH = 25, extend = 1.5, labels = TRUE, ..
     shape = xi
     scale = beta * invProb^xi
     location = threshold - (scale*(invProb^(- xi)-1))/xi
-    
+
     # Show Plot:
-    # Plot
-    plot(sort(data), ypoints, xlim = range(plotmin, plotmax), 
-         ylim = range(ypoints, y, na.rm = TRUE), col = "steelblue",
+    xlim = range(plotmin, plotmax)
+    ylim = range(ypoints, y[y>0], na.rm = TRUE)
+    plot(sort(data), ypoints, xlim = xlim, ylim = ylim, col = "steelblue",
          pch = 19, xlab = "", ylab = "", log = "xy", axes = TRUE, ...)
     lines(z[y >= 0], y[y >= 0])
-    grid()
+    if (grid) grid()
+
     # Labels:
     alog = "xy"
     if (labels) {
@@ -665,40 +668,40 @@ function(object, p = 0.99, ci = 0.95, nLLH = 25, extend = 1.5, labels = TRUE, ..
         title(xlab = xLab, ylab = yLab)
         title(main = "Tail Estimate Plot")
     }
-    
+
     # Object:
     object@fit$n = length(x)
     object@fit$data = object@data$exceedances
     object@fit$n.exceed = length(object@fit$data)
-   
+
     # Tail Plot:
     lastfit = object@fit
     x = list(lastfit = lastfit, type = "tail", dist = "gpd",
-         plotmin = plotmin, plotmax = plotmax, alog = "xy", 
+         plotmin = plotmin, plotmax = plotmax, alog = "xy",
          location = location, shape = shape, scale = scale)
-        
+
     threshold = lastfit$threshold
     par.ests = lastfit$par.ests
     xihat = par.ests["xi"]
     betahat = par.ests["beta"]
     varcov = lastfit$varcov
     p.less.thresh = lastfit$p.less.thresh
-    
+
     par(new = TRUE)
-   
+
     # GPD Quantiles:
     a = 1/(1 - p.less.thresh) * (1 - pp)
     gfunc = function(a, xihat) (a^(-xihat) - 1)/xihat
-    gfunc.deriv = function(a, xihat) 
+    gfunc.deriv = function(a, xihat)
         (-(a^(-xihat)-1)/xihat - a^(-xihat)*logb(a))/xihat
     q = q.keep = threshold + betahat * gfunc(a, xihat)
     # if (q < x$plotmax) abline(v = q, lty = 2)
     out1 = as.numeric(q)
     # Log Likelihood:
     parloglik = function(theta, tmp, a, threshold, xpi) {
-        beta = (theta * (xpi - threshold))/(a^(-theta) - 
+        beta = (theta * (xpi - threshold))/(a^(-theta) -
             1)
-        if ((beta <= 0) || ((theta <= 0) && (max(tmp) > (-beta/theta)))) 
+        if ((beta <= 0) || ((theta <= 0) && (max(tmp) > (-beta/theta))))
             f = 1e+06
         else {
             y = logb(1 + (theta * tmp)/beta)
@@ -711,13 +714,13 @@ function(object, p = 0.99, ci = 0.95, nLLH = 25, extend = 1.5, labels = TRUE, ..
     # x Value:
     theta = xihat
     parmax = NULL
-    xp = exp(seq(from = logb(threshold), to = logb(x$plotmax), 
+    xp = exp(seq(from = logb(threshold), to = logb(x$plotmax),
         length = like.num))
     # y Value:
     excess = as.numeric(x$lastfit$data - threshold)
     for (i in 1:length(xp)) {
-        fit2 = optim(theta, parloglik, method = "BFGS", 
-            hessian = FALSE, tmp = excess, a = a, threshold = threshold, 
+        fit2 = optim(theta, parloglik, method = "BFGS",
+            hessian = FALSE, tmp = excess, a = a, threshold = threshold,
             xpi = xp[i])
         parmax = rbind(parmax, fit2$value)
     }
@@ -729,10 +732,10 @@ function(object, p = 0.99, ci = 0.95, nLLH = 25, extend = 1.5, labels = TRUE, ..
     parmax = parmax[cond]
     # Plot:
     par(new = TRUE)
-    plot(xp, parmax, type = "n", xlab = "", ylab = "", axes = FALSE, 
-        xlim = range(x$plotmin, x$plotmax), 
+    plot(xp, parmax, type = "n", xlab = "", ylab = "", axes = FALSE,
+        xlim = range(x$plotmin, x$plotmax),
         ylim = range(overallmax, crit), log = "x")
-    axis(4, at = overallmax - qchisq(c(0.95, 0.99), 1)/2, 
+    axis(4, at = overallmax - qchisq(c(0.95, 0.99), 1)/2,
         labels = c("95", "99"), tick = TRUE)
     aalpha = qchisq(ci.p, 1)
     abline(h = overallmax - aalpha/2, lty = 2, col = 2)
@@ -745,7 +748,7 @@ function(object, p = 0.99, ci = 0.95, nLLH = 25, extend = 1.5, labels = TRUE, ..
     out1 = c(min(ci), q, max(ci))
     names(out1) = c("Lower CI", "Estimate", "Upper CI")
 
-    
+
     # GPD Shortfall:
     a = 1/(1 - p.less.thresh) * (1 - pp)
     gfunc = function(a, xihat) (a^( - xihat) - 1) / xihat
@@ -785,17 +788,17 @@ function(object, p = 0.99, ci = 0.95, nLLH = 25, extend = 1.5, labels = TRUE, ..
     parmax = parmax[cond]
     # Plot:
     par(new = TRUE)
-    plot(xp, parmax, type = "n", xlab = "", ylab = "", axes = FALSE, 
-         xlim = range(x$plotmin, x$plotmax), 
+    plot(xp, parmax, type = "n", xlab = "", ylab = "", axes = FALSE,
+         xlim = range(x$plotmin, x$plotmax),
          ylim = range(overallmax, crit), log = "x")
     axis(4, at = overallmax - qchisq(c(0.95, 0.99), 1)/2,
-         labels = c("95", "99"), tick = TRUE)   
+         labels = c("95", "99"), tick = TRUE)
     aalpha = qchisq(ci.p, 1)
     abline(h = overallmax - aalpha/2, lty = 2, col = 2)
     cond = !is.na(xp) & !is.na(parmax)
     smth = spline(xp[cond], parmax[cond], n = 200)
     lines(smth, lty = 2, col = 2)
-    ci = smth$x[smth$y > overallmax - aalpha/2]    
+    ci = smth$x[smth$y > overallmax - aalpha/2]
     abline(v = s.keep, lty = 2)
     # Result:
     out2 = c(min(ci), s, max(ci))
@@ -817,8 +820,8 @@ function(object, p = 0.99, ci = 0.95, nLLH = 25, extend = 1.5, labels = TRUE, ..
 .tailSlider.counter = NA
 .tailSlider.Thresholds = NA
 
-    
-tailSlider = 
+
+tailSlider =
 function(x)
 {   # A function implemented by Diethelm Wuertz
 
@@ -827,14 +830,14 @@ function(x)
 
     # Arguments:
     #   x - an univariate timeSeries object or any other object which
-    #       can be transformed by the function as.vector() into a 
+    #       can be transformed by the function as.vector() into a
     #       numeric vector.
-    
+
     # FUNCTION:
-    
+
     # Transform to Vector:
     x = as.vector(x)
-    
+
     # Exit:
     on.exit(rm(.tailSlider.last.Quantile))
     on.exit(rm(.tailSlider.last.nThresholds))
@@ -842,11 +845,11 @@ function(x)
     on.exit(rm(.tailSlider.conf))
     on.exit(rm(.tailSlider.counter))
     on.exit(rm(x))
-    
+
     # Internal Function:
     refresh.code = function(...)
     {
-        .tailSlider.counter <<- .tailSlider.counter + 1      
+        .tailSlider.counter <<- .tailSlider.counter + 1
         # Sliders:
         u = thresholdStart = .sliderMenu(no = 1)
         du = .sliderMenu(no = 2)
@@ -854,23 +857,23 @@ function(x)
         nThresholds = .sliderMenu(no = 4)
         Quantile = .sliderMenu(no = 5)
         pp = .sliderMenu(no = 6)
-          
-        
+
+
         if (.tailSlider.counter > 5) {
-            
-        # Plot data:        
+
+        # Plot data:
         par(mfrow = c(2, 2), cex = 0.7)
 
         # Figure 1:
-        ans = mxfPlot(x, u = quantile(x, 1), 
-            xlim = c(min(x), max.x), labels = FALSE) 
+        ans = mxfPlot(x, u = quantile(x, 1),
+            xlim = c(min(x), max.x), labels = FALSE)
         grid()
-        
+
         # Add thresholds:
-        U = min(c(u+du, max(x)))  
+        U = min(c(u+du, max(x)))
         abline(v = u, lty = 3, col = "red")
         abline(v = U, lty = 3, col = "red")
- 
+
         # Fit line to mean excess within thresolds:
         X = as.vector(ans[, 1])
         Y = as.vector(ans[, 2])
@@ -886,26 +889,26 @@ function(x)
         beta = c(beta = c/(1+m))
         Xi = signif(xi, 3)
         Beta = signif(beta, 3)
-        
+
         # Add Title:
         Main = paste("Fig 1:  xi = ", Xi, "| beta =", Beta)
-        title(main = Main, xlab = "Threshold", ylab = "Mean Excess")   
-        
+        title(main = Main, xlab = "Threshold", ylab = "Mean Excess")
+
         # GPD Fit:
         if (.tailSlider.last.Quantile != Quantile | .tailSlider.last.nThresholds != nThresholds) {
             .tailSlider.param <<- NULL
             .tailSlider.conf <<- NULL
-            .tailSlider.Thresholds <<- seq(quantile(x, Quantile), quantile(x, 1-Quantile), 
+            .tailSlider.Thresholds <<- seq(quantile(x, Quantile), quantile(x, 1-Quantile),
                 length = nThresholds)
             for (threshold in .tailSlider.Thresholds) {
-                ans = gpdFit(x, threshold)@fit 
+                ans = gpdFit(x, threshold)@fit
                 .tailSlider.param <<- rbind(.tailSlider.param, c(u = threshold, ans$par.ests))
                 .tailSlider.conf <<- rbind(.tailSlider.conf, c(u = threshold, ans$par.ses))
             }
             .tailSlider.last.Quantile <<- Quantile
             .tailSlider.last.nThresholds <<- nThresholds
-        } 
-        
+        }
+
         # Figure 2:
         ymax = max(c(.tailSlider.param[, 2] + .tailSlider.conf[, 2]))
         ymin = min(c(.tailSlider.param[, 2] - .tailSlider.conf[, 2]))
@@ -916,13 +919,13 @@ function(x)
         points(.tailSlider.Thresholds, .tailSlider.param[, 2], pch = 19, col = "steelblue")
         lines(.tailSlider.Thresholds, .tailSlider.param[, 2] + .tailSlider.conf[, 2], lty = 3)
         lines(.tailSlider.Thresholds, .tailSlider.param[, 2] - .tailSlider.conf[, 2], lty = 3)
-        abline(h = xi, lty = 3, col = "red") 
+        abline(h = xi, lty = 3, col = "red")
         abline(v = u, lty = 3, col = "red")
         abline(v = U, lty = 3, col = "red")
-          
-        # Figure 3:  
+
+        # Figure 3:
         ymax = max(c(.tailSlider.param[, 3] + .tailSlider.conf[, 3]))
-        ymin = min(c(.tailSlider.param[, 3] - .tailSlider.conf[, 3]))    
+        ymin = min(c(.tailSlider.param[, 3] - .tailSlider.conf[, 3]))
         plot(.tailSlider.Thresholds, .tailSlider.param[, 3], xlab = "Threshold", ylab = "beta",
             ylim = c(ymin, ymax), col = "steelblue", type = "l",
             main = "beta Estimation")
@@ -932,46 +935,46 @@ function(x)
         lines(.tailSlider.Thresholds, .tailSlider.param[, 3] - .tailSlider.conf[, 3], lty = 3)
         abline(h = beta, lty = 3, col = "red")
         abline(v = u, lty = 3, col = "red")
-        abline(v = U, lty = 3, col = "red") 
-        
+        abline(v = U, lty = 3, col = "red")
+
         # Figure 4:
-        #   <<- 
+        #   <<-
         fit = gpdFit(x, u)
         tailPlot(object = fit, p = pp)
-           
+
         # Refresh Frame:
         par(mfrow = c(2, 2), cex = 0.7)
         }
     }
-    
+
     # Save x globally:
     x <<- as.vector(x)
-    
+
     # Slider Menu - x Series Settings:
     xmax = max(x)
     delta.x = (max(x)-min(x))/200
     start.x = par()$usr[2]
-    
+
     # Slider Menu -  Threshold/Quantiles Settings:
     qmin = quantile(x, 0.25)
     qmax = quantile(x, 0.995)
     delta.q = (qmax-qmin)/200
     start.q = (qmin+qmax)/2
-    
+
     # Save Globally:
     .tailSlider.last.Quantile <<- 0.05*(1+1e-4)
     .tailSlider.last.nThresholds <<- 10+1
     .tailSlider.param <<- NA
     .tailSlider.conf <<- NA
     .tailSlider.counter <<- 0
-    
+
     # Open Slider Menu:
     .sliderMenu(refresh.code,
        names =       c("1 thresholdStart",
                                  "1 thresholdInterval",
-                                           "1 max(x)", 
+                                           "1 max(x)",
                                                       "2|3 nThresholds",
-                                                           "2|3 Quantile", 
+                                                           "2|3 Quantile",
                                                                       "pp"),
        minima =      c( qmin,    0,        min(x),    5,    0.005,    0.900),
        maxima =      c( qmax,    qmax,     max(x),    50,   0.500,    0.999),
@@ -983,35 +986,35 @@ function(x)
 # ------------------------------------------------------------------------------
 
 
-tailRisk = 
+tailRisk =
 function(object, prob = c(0.99, 0.995, 0.999, 0.9995, 0.9999), ...)
 {   # A function implemented by Diethelm Wuertz
 
     # Description:
     #   Calculates Quantiles VaR and Expected Shortfall Risks
-    
+
     # Arguments:
     #   x  - an object of class 'gpdFit'
-    #   prob - a numeric value or vector of probability levels 
-    
+    #   prob - a numeric value or vector of probability levels
+
     # FUNCTION:
-    
+
     # Settings:
     u = object@parameter$u
     par.ests = object@fit$par.ests
     xi = par.ests["xi"]
     beta = par.ests["beta"]
     lambda = 1/(1 - object@fit$prob)
-    
-    # Quantile Risk:
-    q = u + (beta * ((lambda * (1 - prob))^( - xi) - 1))/xi 
 
-    # Shortfall Risk:  
-    es = (q * (1 + (beta - xi * u)/q)) / (1 - xi) 
-  
+    # Quantile Risk:
+    q = u + (beta * ((lambda * (1 - prob))^( - xi) - 1))/xi
+
+    # Shortfall Risk:
+    es = (q * (1 + (beta - xi * u)/q)) / (1 - xi)
+
     # Risk Matrix:
     ans = data.frame(Prob = prob, VaR = q, ES = es)
-    
+
     # Return Value:
     ans
 }
